@@ -181,7 +181,9 @@ mod imp {
 
             let obj = self.obj();
 
-            let camerabin = gst::ElementFactory::make("camerabin").build().unwrap();
+            let camerabin = gst::ElementFactory::make("camerabin")
+                .build()
+                .expect("Missing GStreamer Bad Plug-ins");
             self.camerabin.set(camerabin.clone()).unwrap();
 
             let bus = self.camerabin().bus().unwrap();
@@ -208,10 +210,12 @@ mod imp {
                 gst::ElementFactory::make("glsinkbin")
                     .property("sink", &paintablesink)
                     .build()
-                    .unwrap()
+                    .expect("Missing GStreamer Base Plug-ins")
             } else {
                 let bin = gst::Bin::default();
-                let convert = gst::ElementFactory::make("videoconvert").build().unwrap();
+                let convert = gst::ElementFactory::make("videoconvert")
+                    .build()
+                    .expect("Missing GStreamer Base Plug-ins");
 
                 bin.add(&convert).unwrap();
                 bin.add(&paintablesink).unwrap();
@@ -242,6 +246,16 @@ mod imp {
 
             let devices = crate::DeviceProvider::instance();
 
+            self.devices.set(devices.clone()).unwrap();
+
+            if devices.started() {
+                obj.init();
+            } else {
+                devices.connect_started_notify(glib::clone!(@weak obj => move |_| {
+                    obj.init();
+                }));
+            }
+
             devices.connect_camera_added(glib::clone!(@weak obj => move |_, camera| {
                 if matches!(obj.state(), ViewfinderState::NoCameras | ViewfinderState::Loading | ViewfinderState::Error) {
                     obj.imp().set_state(ViewfinderState::Ready);
@@ -263,24 +277,8 @@ mod imp {
                 }
             }));
 
-            if let Some(camera) = devices.camera(0) {
-                obj.imp().set_state(ViewfinderState::Ready);
-                obj.set_camera(Some(camera));
-            }
-
-            self.devices.set(devices.clone()).unwrap();
-
             log::debug!("Setup recording");
             obj.setup_recording();
-
-            glib::timeout_add_local_once(
-                std::time::Duration::from_secs(PROVIDER_TIMEOUT),
-                glib::clone!(@weak obj => move || {
-                    if matches!(obj.state(), ViewfinderState::Loading) {
-                        obj.imp().set_state(ViewfinderState::NoCameras);
-                    }
-                }),
-            );
         }
 
         fn dispose(&self) {
@@ -759,6 +757,30 @@ impl Viewfinder {
 
         camerabin.set_property("video-profile", profiles);
     }
+
+    fn init(&self) {
+        let imp = self.imp();
+        let devices = imp.devices.get().unwrap();
+
+        if let Some(camera) = devices.default_camera().or_else(|| devices.camera(0)) {
+            if matches!(
+                self.state(),
+                ViewfinderState::NoCameras | ViewfinderState::Loading | ViewfinderState::Error
+            ) {
+                imp.set_state(ViewfinderState::Ready);
+                self.set_camera(Some(camera));
+            }
+        }
+
+        glib::timeout_add_local_once(
+            std::time::Duration::from_secs(PROVIDER_TIMEOUT),
+            glib::clone!(@weak self as obj => move || {
+                if matches!(obj.state(), ViewfinderState::Loading) {
+                    obj.imp().set_state(ViewfinderState::NoCameras);
+                }
+            }),
+        );
+    }
 }
 
 fn create_zbar_bin() -> Result<gst::Element, glib::BoolError> {
