@@ -1,10 +1,9 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 use adw::prelude::*;
+use adw::subclass::prelude::*;
 use gettextrs::gettext;
-use gtk::subclass::prelude::*;
 use gtk::CompositeTemplate;
-use gtk::{gio, glib};
-
+use gtk::{gdk, gio, glib};
 use once_cell::sync::Lazy;
 
 static ATTRIBUTES: Lazy<String> = Lazy::new(|| {
@@ -19,20 +18,29 @@ static ATTRIBUTES: Lazy<String> = Lazy::new(|| {
 });
 
 mod imp {
-    use super::*;
+    use std::cell::RefCell;
 
     use once_cell::sync::Lazy;
-    use std::cell::RefCell;
+
+    use super::*;
 
     #[derive(Debug, Default, CompositeTemplate)]
     #[template(resource = "/org/gnome/Snapshot/ui/gallery.ui")]
     pub struct Gallery {
         #[template_child]
-        pub child: TemplateChild<gtk::Widget>,
+        pub mobile_breakpoint: TemplateChild<adw::Breakpoint>,
         #[template_child]
         pub sliding_view: TemplateChild<crate::SlidingView>,
         #[template_child]
         pub open_external: TemplateChild<gtk::Button>,
+        #[template_child]
+        pub desktop_controls: TemplateChild<gtk::MediaControls>,
+        #[template_child]
+        pub desktop_controls_revealer: TemplateChild<gtk::Revealer>,
+        #[template_child]
+        pub mobile_controls: TemplateChild<gtk::MediaControls>,
+        #[template_child]
+        pub menu_button: TemplateChild<gtk::MenuButton>,
 
         pub current_item: RefCell<Option<crate::GalleryItem>>,
     }
@@ -41,11 +49,10 @@ mod imp {
     impl ObjectSubclass for Gallery {
         const NAME: &'static str = "Gallery";
         type Type = super::Gallery;
-        type ParentType = gtk::Widget;
+        type ParentType = adw::BreakpointBin;
 
         fn class_init(klass: &mut Self::Class) {
             klass.bind_template();
-            klass.set_layout_manager_type::<gtk::BinLayout>();
             klass.set_css_name("gallery");
 
             // Shows an older picture (scrolls to the right)
@@ -61,6 +68,16 @@ mod imp {
                     log::error!("Could not open with system handler: {err}");
                 }
             });
+            klass.install_action("gallery.copy", None, |widget, _, _| {
+                if let Err(err) = widget.copy() {
+                    log::error!("Could not copy gallery item: {err}");
+                }
+            });
+            klass.install_action_async("gallery.delete", None, |widget, _, _| async move {
+                if let Err(err) = widget.delete().await {
+                    log::error!("Could not delete gallery item: {err}");
+                }
+            });
         }
 
         fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
@@ -69,10 +86,6 @@ mod imp {
     }
 
     impl ObjectImpl for Gallery {
-        fn dispose(&self) {
-            self.dispose_template();
-        }
-
         fn constructed(&self) {
             self.parent_constructed();
 
@@ -80,24 +93,35 @@ mod imp {
 
             self.sliding_view.connect_target_page_reached(
                 glib::clone!(@weak obj => move |sliding_view| {
-                    let imp = obj.imp();
-
-                    if let Some(current) = sliding_view.current_page() {
-                        // The tooltip is also set in gallery.ui with a default value.
-                        let tooltip_text = if current.is_picture() {
-                            gettext("Open in Image Viewer")
-                        } else {
-                            gettext("Open in Video Player")
-                        };
-                        imp.open_external.set_tooltip_text(Some(&tooltip_text));
-                    }
-
                     obj.load_neighbor_pages();
+
+                    // When deleting an item we need to check again.
+                    let has_prev = sliding_view.prev_page().is_some();
+                    let has_next = sliding_view.next_page().is_some();
+                    obj.action_set_enabled("gallery.previous", has_prev);
+                    obj.action_set_enabled("gallery.next", has_next);
                 }),
             );
 
             self.sliding_view.connect_current_page_notify(
                 glib::clone!(@weak obj => move |sliding_view| {
+                    let imp = obj.imp();
+
+                    if let Some(current) = sliding_view.current_page() {
+                        let is_picture = current.is_picture();
+
+                        // The tooltip is also set in gallery.ui with a default value.
+                        let tooltip_text = if is_picture {
+                            gettext("Open in Image Viewer")
+                        } else {
+                            gettext("Open in Video Player")
+                        };
+                        imp.open_external.set_tooltip_text(Some(&tooltip_text));
+
+                        let menu = crate::utils::gallery_item_menu(is_picture);
+                        imp.menu_button.set_menu_model(Some(&menu));
+                    }
+
                     let has_prev = sliding_view.prev_page().is_some();
                     let has_next = sliding_view.next_page().is_some();
                     obj.action_set_enabled("gallery.previous", has_prev);
@@ -108,9 +132,13 @@ mod imp {
                             video.pause();
                         }
                     }
+
+                    obj.setup_media_controls();
                 }),
             );
 
+            obj.setup_media_controls();
+
             let ctx = glib::MainContext::default();
             ctx.spawn_local(glib::clone!(@weak obj => async move {
                 if let Err(err) = obj.load_items().await {
@@ -121,19 +149,25 @@ mod imp {
 
         fn signals() -> &'static [glib::subclass::Signal] {
             static SIGNALS: Lazy<Vec<glib::subclass::Signal>> = Lazy::new(|| {
-                vec![glib::subclass::Signal::builder("item-added")
-                    .param_types([crate::GalleryItem::static_type()])
-                    .build()]
+                vec![
+                    glib::subclass::Signal::builder("item-added")
+                        .param_types([crate::GalleryItem::static_type()])
+                        .build(),
+                    glib::subclass::Signal::builder("item-removed")
+                        .param_types([bool::static_type()])
+                        .build(),
+                ]
             });
             SIGNALS.as_ref()
         }
     }
     impl WidgetImpl for Gallery {}
+    impl BreakpointBinImpl for Gallery {}
 }
 
 glib::wrapper! {
     pub struct Gallery(ObjectSubclass<imp::Gallery>)
-        @extends gtk::Widget;
+        @extends gtk::Widget, adw::BreakpointBin;
 }
 
 impl Default for Gallery {
@@ -177,10 +211,6 @@ impl Gallery {
         }
     }
 
-    pub fn close(&self) {
-        // TODO
-    }
-
     pub fn items(&self) -> Vec<crate::GalleryItem> {
         self.imp().sliding_view.pages()
     }
@@ -189,6 +219,10 @@ impl Gallery {
         self.emit_by_name::<()>("item-added", &[&picture]);
     }
 
+    fn emit_item_removed(&self, is_last: bool) {
+        self.emit_by_name::<()>("item-removed", &[&is_last]);
+    }
+
     pub fn connect_item_added<F: Fn(&Self, &crate::GalleryItem) + 'static>(&self, f: F) {
         self.connect_closure(
             "item-added",
@@ -199,6 +233,16 @@ impl Gallery {
         );
     }
 
+    pub fn connect_item_removed<F: Fn(&Self, bool) + 'static>(&self, f: F) {
+        self.connect_closure(
+            "item-removed",
+            false,
+            glib::closure_local!(|obj, is_last| {
+                f(obj, is_last);
+            }),
+        );
+    }
+
     fn next(&self) {
         let imp = self.imp();
         if let Some(page) = imp.sliding_view.next_page() {
@@ -247,7 +291,9 @@ impl Gallery {
             .next_files_future(1, glib::Priority::default())
             .await
         {
-            let Some(file_info) = info.first() else { break; };
+            let Some(file_info) = info.first() else {
+                break;
+            };
 
             let name = file_info.name();
             let file = gio::File::for_path(&dir.join(&name));
@@ -325,4 +371,77 @@ impl Gallery {
             }
         }
     }
+
+    fn copy(&self) -> anyhow::Result<()> {
+        let imp = self.imp();
+
+        if let Some(item) = imp.sliding_view.current_page() {
+            let file = item.file();
+            let list = gdk::FileList::from_array(&[file]);
+            let provider = gdk::ContentProvider::for_value(&list.to_value());
+
+            self.clipboard().set_content(Some(&provider))?;
+
+            let window = self.root().and_downcast::<crate::Window>().unwrap();
+            window.send_toast(&gettext("Copied to clipboard"));
+
+            Ok(())
+        } else {
+            anyhow::bail!("Sliding view does not currently have a page");
+        }
+    }
+
+    fn setup_media_controls(&self) {
+        let imp = self.imp();
+
+        if let Some(video) = imp
+            .sliding_view
+            .current_page()
+            .and_downcast_ref::<crate::GalleryVideo>()
+        {
+            imp.mobile_controls.set_media_stream(Some(video.stream()));
+            imp.desktop_controls.set_media_stream(Some(video.stream()));
+
+            imp.mobile_controls.set_visible(true);
+            imp.desktop_controls_revealer.set_reveal_child(true);
+        } else {
+            imp.mobile_controls.set_media_stream(gtk::MediaStream::NONE);
+            imp.desktop_controls
+                .set_media_stream(gtk::MediaStream::NONE);
+
+            imp.mobile_controls.set_visible(false);
+            imp.desktop_controls_revealer.set_reveal_child(false);
+        }
+    }
+
+    async fn delete(&self) -> anyhow::Result<()> {
+        let imp = self.imp();
+
+        if let Some(item) = imp.sliding_view.current_page() {
+            let is_last = imp
+                .sliding_view
+                .pages()
+                .first()
+                .is_some_and(|page| page == &item);
+
+            imp.sliding_view.remove(&item);
+
+            let file = item.file();
+            file.delete_future(glib::Priority::default()).await?;
+
+            let window = self.root().and_downcast::<crate::Window>().unwrap();
+            if item.is_picture() {
+                window.send_toast(&gettext("Picture deleted"));
+            } else {
+                window.send_toast(&gettext("Video deleted"));
+            }
+            self.load_neighbor_pages();
+
+            self.emit_item_removed(is_last);
+
+            Ok(())
+        } else {
+            anyhow::bail!("Sliding view does not currently have a page");
+        }
+    }
 }
