@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-3.0-or-later
 use adw::prelude::*;
 use adw::subclass::prelude::*;
 use gtk::CompositeTemplate;
@@ -6,9 +7,9 @@ use gtk::{gio, glib};
 use crate::config;
 
 mod imp {
-    use super::*;
+    use std::cell::OnceCell;
 
-    use once_cell::sync::OnceCell;
+    use super::*;
 
     #[derive(Debug, Default, CompositeTemplate)]
     #[template(resource = "/org/gnome/Snapshot/ui/preferences_window.ui")]
@@ -49,10 +50,6 @@ mod imp {
 
             self.settings.set(settings).unwrap();
         }
-
-        fn dispose(&self) {
-            self.dispose_template();
-        }
     }
 
     impl WidgetImpl for PreferencesWindow {}
