@@ -20,11 +20,11 @@ const HOVER_SCALE: f64 = 1.05;
 const HOVER_DURATION: u32 = 125;
 
 mod imp {
-    use super::*;
+    use std::cell::{Cell, OnceCell};
 
     use glib::Properties;
-    use once_cell::sync::OnceCell;
-    use std::cell::Cell;
+
+    use super::*;
 
     #[derive(Debug, Properties, Default)]
     #[properties(wrapper_type = super::ShutterButton)]
@@ -105,19 +105,8 @@ mod imp {
         type ParentType = gtk::Button;
     }
 
+    #[glib::derived_properties]
     impl ObjectImpl for ShutterButton {
-        fn properties() -> &'static [glib::ParamSpec] {
-            Self::derived_properties()
-        }
-
-        fn property(&self, id: usize, pspec: &glib::ParamSpec) -> glib::Value {
-            Self::derived_property(self, id, pspec)
-        }
-
-        fn set_property(&self, id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
-            Self::derived_set_property(self, id, value, pspec)
-        }
-
         fn constructed(&self) {
             self.parent_constructed();
 
@@ -159,6 +148,7 @@ mod imp {
             let countdown_ani = adw::TimedAnimation::new(&*widget, 1.0, 0.0, 250, countdown_target);
             // TODO Figure out what easing to use.
             countdown_ani.set_easing(adw::Easing::Linear);
+            countdown_ani.set_follow_enable_animations_setting(false);
             self.countdown_ani.set(countdown_ani).unwrap();
 
             let record_target =
@@ -379,7 +369,7 @@ fn lerp(t: f32, from: f32, to: f32) -> f32 {
 fn color_lerp(t: f32, from: gdk::RGBA, to: gdk::RGBA) -> gdk::RGBA {
     gdk::RGBA::new(
         lerp(t, from.red(), to.red()),
-        lerp(t, from.green(), to.blue()),
+        lerp(t, from.green(), to.green()),
         lerp(t, from.blue(), to.blue()),
         from.alpha(),
     )
